package czerwony.kapturek.jbwheel;

import java.util.*;
import java.io.*;

import com.borland.primetime.properties.PropertyManager;
import com.borland.primetime.vfs.VFS;

public class Config
{
    // following keys are defined as in Windows
    public final static int	MK_LBUTTON	= 0x01;
    public final static int	MK_RBUTTON	= 0x02;
    public final static int	MK_SHIFT	= 0x04;
    public final static int	MK_CONTROL	= 0x08;
    public final static int	MK_MBUTTON	= 0x10;
    // Alt key has no Windows MK_ code
    public final static int	MK_ALT		= 0x8000;

    public static final int	MODE_EDITORONLY		= 1;
    public static final int	MODE_FOCUSED		= 2;
    public static final int	MODE_UNDERCURSOR	= 3;

    private static final String	KEY_SENSITIVITY	    = "scroll.sensitivity";
    private static final String	KEY_MODE		    = "scroll.mode";
    private static final String	KEY_FOLLOWBAR	    = "scroll.followscrollbar";
    private static final String	KEY_BOTHDIRECTIONS  = "scroll.bothdirections";
    private static final String	KEY_REVERSE		    = "reverse";
    private static final String	KEY_ACTION		    = "action";
    private static final String	KEY_DEBUG		    = "debug";

    private static final String FNAME = "jbWheel.properties";

    private static final int		DEF_SENSITIVITY	    = 30;
    private static final int		DEF_MODE		    = MODE_UNDERCURSOR;
    private static final boolean	DEF_REVERSE		    = false;
    private static final boolean	DEF_FOLLOWBAR	    = true;
    private static final boolean	DEF_DEBUG		    = false;
    private static final boolean	DEF_BOTHDIRECTIONS  = true;

    public int		sensitivity	= DEF_SENSITIVITY;
    public int		mode		= DEF_MODE;
    public boolean	reverse 	= DEF_REVERSE;
    /** useful only in "component at cursor" mode: scroll in the direction
      of the scrollbar under cursor, no matter what direction is mapped */
    public boolean	followBar 	= DEF_FOLLOWBAR;
    public HashMap	actions		= new HashMap();
    public boolean	debug		= DEF_DEBUG;
    public boolean  bothDirections = DEF_BOTHDIRECTIONS;

    private Properties props = new Properties();

    public Config( Config c )
    {
        this.sensitivity = c.sensitivity;
        this.mode = c.mode;
        this.reverse = c.reverse;
        this.followBar = c.followBar;
        this.actions = c.actions;
        this.debug = c.debug;
        this.bothDirections = c.bothDirections;
    }

    public Config()
    {
        read();
        if ( actions.isEmpty() )
        {
            actions.put( new Integer(0),			Action.VSCROLL );
            actions.put( new Integer(MK_CONTROL),	Action.PGVSCROLL );
            actions.put( new Integer(MK_SHIFT),		Action.NEXTNODE );
        }
    }

    public boolean read()
    {
        try
        {
            BufferedInputStream in = new BufferedInputStream(
                    VFS.getInputStream( PropertyManager.getSettingsUrl(FNAME)));
            try
            {
                props.load( in );
                sensitivity = Integer.parseInt(
                    props.getProperty( KEY_SENSITIVITY, String.valueOf(DEF_SENSITIVITY) ) );
                mode = Integer.parseInt(
                    props.getProperty( KEY_MODE, String.valueOf(DEF_MODE) ) );
                reverse = Boolean.valueOf(
                    props.getProperty( KEY_REVERSE, String.valueOf(DEF_REVERSE) ) ).booleanValue();
                followBar = Boolean.valueOf(
                    props.getProperty( KEY_FOLLOWBAR, String.valueOf(DEF_FOLLOWBAR) ) ).booleanValue();
                bothDirections = Boolean.valueOf(
                    props.getProperty( KEY_BOTHDIRECTIONS, String.valueOf(DEF_BOTHDIRECTIONS) ) ).booleanValue();
                debug = Boolean.valueOf(
                    props.getProperty( KEY_DEBUG, String.valueOf(DEF_DEBUG) ) ).booleanValue();

                Enumeration en = props.keys();
                while ( en.hasMoreElements() )
                {
                    String key = (String)en.nextElement();
                    if ( key.startsWith( KEY_ACTION ) )
                    {
                        String a = props.getProperty( key );
                        key = key.substring( KEY_ACTION.length()+1 );
                        actions.put( Integer.decode( key ), Integer.decode( a ) );
                    }
                }
            }
            finally
            {
                in.close();
            }
        }
        catch( Exception ex )
        {
            return false;
        }
        return true;
    }

    public boolean save()
    {
        try
        {
            /*File f = new File( PropertyManager.getSettingsUrl(FNAME).getFile() );
            if ( f.exists() )
            {
                System.err.println( "" + f.delete() );
            }*/
            BufferedOutputStream out = new BufferedOutputStream(
                    VFS.getOutputStream( PropertyManager.getSettingsUrl(FNAME)));
            try
            {
                props.clear();
                props.setProperty( KEY_SENSITIVITY, String.valueOf(sensitivity) );
                props.setProperty( KEY_MODE, String.valueOf(mode) );
                props.setProperty( KEY_REVERSE, String.valueOf(reverse) );
                props.setProperty( KEY_FOLLOWBAR, String.valueOf(followBar) );
                props.setProperty( KEY_BOTHDIRECTIONS, String.valueOf(bothDirections) );
                props.setProperty( KEY_DEBUG, String.valueOf(debug) );

                Iterator i = actions.keySet().iterator();
                while ( i.hasNext() )
                {
                    Object key = i.next();
                    Object a = actions.get( key );
                    props.setProperty( KEY_ACTION + "." + key.toString(), a.toString() );
                    //System.err.println( key.toString() );
                }

                props.store( out, "jbWheel settings" );
            }
            finally
            {
                out.close();
            }
        }
        catch( Exception ex )
        {
            return false;
        }
        return true;
    }

    public static interface Action
    {
        public final static int none		= 0;
        public final static int vscroll		= 1;
        public final static int hscroll		= 2;
        public final static int pgvscroll	= 3;
        public final static int pghscroll	= 5;
        public final static int nextnode	= 4;
        public final static Integer NONE		= new Integer(none);
        public final static Integer VSCROLL		= new Integer(vscroll);
        public final static Integer HSCROLL		= new Integer(hscroll);
        public final static Integer PGVSCROLL	= new Integer(pgvscroll);
        public final static Integer PGHSCROLL	= new Integer(pghscroll);
        public final static Integer NEXTNODE	= new Integer(nextnode);

        public final static String[] names =
            {
                "None",
                "Scroll up/down",
                "Scroll left/right",
                "Page up/down",
                "Go to next/previous file",
                "Page left/right"
            };
    }
}